// Prevent console window in addition to Slint window in Windows release builds when, e.g., starting the app via file manager. Ignored on other platforms.
#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]

use slint::Model;
use std::{cell::RefCell, rc::Rc};

slint::include_modules!();

struct GUIElements {
    tab_vram_file_list: Rc<slint::VecModel<slint::StandardListViewItem>>
}

impl GUIElements {
    pub fn new(main_window: &MainWindow) -> GUIElements {
        let tab_vram_file_list:Vec<slint::StandardListViewItem> = main_window.get_main_tab_vram_file_list().iter().collect();
        let tab_vram_file_list                                  = Rc::new(slint::VecModel::from(tab_vram_file_list));

        main_window.set_main_tab_vram_file_list(tab_vram_file_list.clone().into());
        GUIElements{tab_vram_file_list}
    }

    pub fn add_new_vram_file(&mut self, file: &str) {
        self.tab_vram_file_list.push(slint::StandardListViewItem::from(file));
    }

    pub fn remove_vram_file(&mut self, idx: usize) {
        self.tab_vram_file_list.remove(idx);
    }
}

fn main() -> Result<(), slint::PlatformError> {
    let main_window  = MainWindow::new()?;
    let gui_elements = Rc::new(RefCell::new(GUIElements::new(&main_window)));

    let gui_elements_cloned = gui_elements.clone();
    main_window.on_main_tab_add_file_clicked(move || {
        gui_elements_cloned.borrow_mut().add_new_vram_file("Planschi");
    });
    let gui_elements_cloned = gui_elements.clone();
    main_window.on_main_tab_remove_file_clicked(move |idx: i32| {
        if idx >= 0 {
            gui_elements_cloned.borrow_mut().remove_vram_file(idx as usize);
        }
    });
    main_window.run()
}